@@ -61,17 +61,21 @@
     }
     return self;
 }
+- (void)locationManager:(CLLocationManager *)manager didChangeAuthorizationStatus:(CLAuthorizationStatus)status
+{
+    Q_UNUSED(manager)
+    if (status == kCLAuthorizationStatusNotDetermined)
+        m_positionInfoSource->requestUpdate(MINIMUM_UPDATE_INTERVAL);
+}
 
 - (void)locationManager:(CLLocationManager *)manager didUpdateToLocation:(CLLocation *)newLocation fromLocation:(CLLocation *)oldLocation
 {
-    Q_UNUSED(manager)
-    Q_UNUSED(oldLocation)
+    Q_UNUSED(manager);
+    Q_UNUSED(oldLocation);
 
     // Convert location timestamp to QDateTime
-    QDateTime timeStamp;
     NSTimeInterval locationTimeStamp = [newLocation.timestamp timeIntervalSince1970];
-    timeStamp.setTime_t((uint) locationTimeStamp);
-    timeStamp.setTime(timeStamp.time().addMSecs((uint)(locationTimeStamp * 1000) % 1000));
+    const QDateTime timeStamp = QDateTime::fromMSecsSinceEpoch(qRound64(locationTimeStamp * 1000), Qt::UTC);
 
     // Construct position info from location data
     QGeoPositionInfo location(QGeoCoordinate(newLocation.coordinate.latitude,
@@ -94,7 +98,7 @@
 
 - (void)locationManager:(CLLocationManager *)manager didFailWithError:(NSError *)error
 {
-    Q_UNUSED(manager)
+    Q_UNUSED(manager);
     m_positionInfoSource->setError(QGeoPositionInfoSource::AccessError);
 
     qWarning() << QString::fromNSString([error localizedDescription]);
@@ -200,22 +204,21 @@ bool QGeoPositionInfoSourceCL::enableLocationManager()
         // while probably a noop, the call generates a warning).
         // -requestWhenInUseAuthorization only requires NSLocationWhenInUseUsageDescription
         // entry in Info.plist (available on iOS (>= 8.0), tvOS (>= 9.0) and watchOS (>= 2.0).
+    }
 
 #ifndef Q_OS_MACOS
-        NSDictionary<NSString *, id> *infoDict = NSBundle.mainBundle.infoDictionary;
-        const bool hasAlwaysUseUsage = !![infoDict objectForKey:@"NSLocationAlwaysAndWhenInUseUsageDescription"];
-        const bool hasWhenInUseUsage = !![infoDict objectForKey:@"NSLocationWhenInUseUsageDescription"];
+    NSDictionary<NSString *, id> *infoDict = NSBundle.mainBundle.infoDictionary;
+    const bool hasAlwaysUseUsage = !![infoDict objectForKey:@"NSLocationAlwaysAndWhenInUseUsageDescription"];
+    const bool hasWhenInUseUsage = !![infoDict objectForKey:@"NSLocationWhenInUseUsageDescription"];
 #ifndef Q_OS_TVOS
-        if (hasAlwaysUseUsage && hasWhenInUseUsage)
-            [m_locationManager requestAlwaysAuthorization];
-        else
+    if (hasAlwaysUseUsage && hasWhenInUseUsage)
+        [m_locationManager requestAlwaysAuthorization];
+    else
 #endif // !Q_OS_TVOS
-        if (hasWhenInUseUsage)
-            [m_locationManager requestWhenInUseAuthorization];
+    if (hasWhenInUseUsage)
+        [m_locationManager requestWhenInUseAuthorization];
 #endif // !Q_OS_MACOS
 
-    }
-
     return (m_locationManager != nullptr);
 }
 
@@ -309,7 +312,7 @@ void QGeoPositionInfoSourceCL::locationDataAvailable(QGeoPositionInfo location)
 
 QGeoPositionInfo QGeoPositionInfoSourceCL::lastKnownPosition(bool fromSatellitePositioningMethodsOnly) const
 {
-    Q_UNUSED(fromSatellitePositioningMethodsOnly)
+    Q_UNUSED(fromSatellitePositioningMethodsOnly);
 
     return m_lastUpdate;
 }
@@ -325,6 +328,6 @@ void QGeoPositionInfoSourceCL::setError(QGeoPositionInfoSource::Error positionEr
     emit QGeoPositionInfoSource::error(positionError);
 }
 
-#include "moc_qgeopositioninfosource_cl_p.cpp"
-
 QT_END_NAMESPACE
+
+#include "moc_qgeopositioninfosource_cl_p.cpp"
